package org.nakedobjects.nof.reflect.help;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;


public class SimpleHelpManager implements HelpManager {
    private static final Logger LOG = Logger.getLogger(SimpleHelpManager.class);
    private static final String CLASS_PREFIX = "c:";
    private static final String NAME_PREFIX = "m:";
    private String fileName = "help.txt";

    public String help(final MemberIdentifier identifier) {
        BufferedReader reader = null;
        try {
            reader = getReader();

            if (reader == null) {
                return "No help available (no file found)";
            }

            String className = CLASS_PREFIX + identifier.getClassName().toLowerCase();
            String name = NAME_PREFIX + identifier.getName().toLowerCase();

            StringBuffer str = new StringBuffer();
            String line;

            boolean lookingForClass = true;
            boolean lookingForName = identifier.getName().length() > 0;
            /*
             * Read through each line in file.
             */
            while ((line = reader.readLine()) != null) {
                // Skip comments - lines begining with hash
                if (line.length() > 0 && line.charAt(0) == '#') {
                    continue;
                }

                /*
                 * Look for class.
                 */
                if (line.toLowerCase().equals(className)) {
                    lookingForClass = false;
                    continue;
                }

                if (lookingForClass) {
                    continue;
                } else if (line.toLowerCase().startsWith(CLASS_PREFIX)) {
                    break;
                }

                /*
                 * Look for field/method.
                 */
                if (line.toLowerCase().equals(name)) {
                    lookingForName = false;
                    continue;
                }

                if (lookingForName) {
                    continue;
                } else if (line.toLowerCase().startsWith(NAME_PREFIX)) {
                    break;
                }

                str.append(line);
                str.append('\n');
            }

            return str.toString();

        } catch (FileNotFoundException e) {
            LOG.error("opening help file", e);
            return "Failure opening help file: " + e.getMessage();
        } catch (IOException e) {
            LOG.error("reading help file", e);
            return "Failure reading help file: " + e.getMessage();
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException ignore) {}
            }
        }
    }

    protected BufferedReader getReader() throws FileNotFoundException {
        File file = new File(fileName);
        if (!file.exists()) {
            String message = "No help file found: " + file.getAbsolutePath();
            LOG.warn(message);
            return null;
        }

        return new BufferedReader(new FileReader(file));
    }

    public void setFileName(final String fileName) {
        this.fileName = fileName;
    }

    public void init() {}

    public void shutdown() {}
}
// Copyright (c) Naked Objects Group Ltd.
