package org.nakedobjects.nof.reflect.i18n;

import org.nakedobjects.nof.reflect.peer.MemberIdentifier;
import org.nakedobjects.object.RequiresSetup;


/**
 * Authorises the user in the current session view and use members of an object.
 */
public interface I18nManager extends RequiresSetup {

    /**
     * Get the localized description for the specified identified action/property. Returns null if no
     * description available.
     */
    String getDescription(MemberIdentifier identifier);

    /**
     * Get the localized name for the specified identified action/property. Returns null if no name available.
     */
    String getName(MemberIdentifier identifier);

    /**
     * Get the localized help text for the specified identified action/property. Returns null if no help text
     * available.
     */
    String getHelp(MemberIdentifier identifier);

    /**
     * Get the localized parameter names for the specified identified action/property. Returns null if no
     * parameters are available. Otherwise returns an array of String objects the size of the number of
     * parameters, where each element is the localised name for the corresponding parameter, or is null if no
     * parameter name is available.
     */
    String[] getParameterNames(MemberIdentifier identifier);
}
// Copyright (c) Naked Objects Group Ltd.
