package org.nakedobjects.nof.reflect.peer;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedReference;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.Formatter;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.util.DebugString;


public abstract class AbstractValuePeer implements ValuePeer {
    private final ValuePeer decorated;

    public AbstractValuePeer(final ValuePeer local) {
        this.decorated = local;
    }

    public boolean canClear() {
        return decorated.canClear();
    }
    
    public boolean canWrap() {
        return decorated.canWrap();
    }
    
    public void clearValue(NakedObject inObject) {
        decorated.clearValue(inObject);
    }

    public void debugData(final DebugString debugString) {
        decorated.debugData(debugString);
    }
    
    public String getBusinessKeyName() {
        return decorated.getBusinessKeyName();
    }

    public NakedValue getDefault(NakedObject target) {
        return decorated.getDefault(target);
    }

    public String getDescription() {
        return decorated.getDescription();
    }

    public Object getExtension(final Class cls) {
        return decorated.getExtension(cls);
    }

    public Class[] getExtensions() {
        return decorated.getExtensions();
    }

    public Formatter getFormatter() {
        return decorated.getFormatter();
    }

    public String getHelp() {
        return decorated.getHelp();
    }

    public MemberIdentifier getIdentifier() {
        return decorated.getIdentifier();
    }

    public int getMaximumLength() {
        return decorated.getMaximumLength();
    }

    public String getName() {
        return decorated.getName();
    }

    public int getNoLines() {
        return decorated.getNoLines();
    }

    public Object[] getOptions(NakedReference target) {
        return decorated.getOptions(target);
    }

    public NakedObjectSpecification getSpecification() {
        return decorated.getSpecification();
    }

    public int getTypicalLineLength() {
        return decorated.getTypicalLineLength();
    }

    public NakedValue getValue(final NakedObject inObject) {
        return decorated.getValue(inObject);
    }

    public void initValue(final NakedObject inObject, final NakedValue val) {
        decorated.initValue(inObject, val);
    }

    public boolean isPersisted() {
        return decorated.isPersisted();
    }

    public boolean isEmpty(final NakedObject inObject) {
        return decorated.isEmpty(inObject);
    }

    public boolean isMandatory() {
        return decorated.isMandatory();
    }

    public boolean isOptionEnabled() {
        return decorated.isOptionEnabled();
    }

    public Consent isUsable(final NakedReference target) {
        return decorated.isUsable(target);
    }

    public Consent isUsable(final Session session) {
        Consent usableDeclaratively = isUsableDeclaratively();
        if (usableDeclaratively.isVetoed()) {
            return usableDeclaratively;
        }
        return isUsableForSession(session);
    }

    public Consent isUsableDeclaratively() {
        return decorated.isUsableDeclaratively();
    }

    public Consent isUsableForSession(
            Session session) {
        return decorated.isUsableForSession(session);
    }


    public Consent isValueValid(final NakedObject inObject, final NakedValue value) {
        return decorated.isValueValid(inObject, value);
    }

    public boolean isVisibleDeclaratively() {
        return decorated.isVisibleDeclaratively();
    }
    
    public boolean isVisibleForSession(final Session session) {
        return decorated.isVisibleForSession(session);
    }
    
    public boolean isVisible(final NakedReference target) {
        return decorated.isVisible(target);
    }

    public void setValue(final NakedObject inObject, final NakedValue val) {
        decorated.setValue(inObject, val);
    }
}
// Copyright (c) Naked Objects Group Ltd.
