package org.nakedobjects.nof.reflect.peer;

import org.nakedobjects.noa.adapter.NakedReference;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.noa.spec.NamedAndDescribedExtensionHolder;
import org.nakedobjects.nof.core.util.DebugString;


/**
 * Details about action and field members gained via reflection.
 * 
 * @see org.nakedobjects.nof.reflect.peer.ActionPeer
 * @see org.nakedobjects.nof.reflect.peer.FieldPeer
 */
public interface MemberPeer extends NamedAndDescribedExtensionHolder {
    
    void debugData(DebugString debugString);

    MemberIdentifier getIdentifier();


    /**
     * Return the help text for this member - the field or action - to complement the description. May return
     * null if no help text exists, or cannot be got reflectively
     * 
     * @see #getDescription()
     */
    String getHelp();

    /**
     * Determines whether this method is usable declaratively.
     */
    Consent isUsableDeclaratively();

    /**
     * Determines whether this method is usable for the specified user.
     */
    Consent isUsableForSession(Session session);

    /**
     * Determines whether this method is usable on, or the field is editable within, the specified object.
     */
    Consent isUsable(NakedReference target);

    
    /**
     * Determines if the member is visible declaratively.
     * 
     * @see #isVisible(NakedReference)
     * @see NakedObjectField#isHidden()
     */
    boolean isVisibleDeclaratively();
    
    /**
     * Determines if the user is authorised to access this member, and hence whether it is visible.
     * 
     * @see #isVisible(NakedReference)
     * @see NakedObjectField#isHidden()
     */
    boolean isVisibleForSession(Session session);

    /**
     * Determines if this member is visible according to the current state of the object.
     * 
     * @see #isVisible(Session)
     */
    boolean isVisible(NakedReference target);
    
}
// Copyright (c) Naked Objects Group Ltd.
