package org.nakedobjects.nof.reflect.remote.data;

import java.util.Vector;

import org.nakedobjects.noa.adapter.NakedObject;

/**
 * Encapsulates a transaction occurring on the client, where each of the actions (add, remove and change) are
 * then passed to the server as an atomic unit.  Each action is captured as an ClientSideTransaction.Entry object.
 */
public class ClientSideTransaction {
    private final Vector transactionEvents = new Vector();

    private void add(final ClientTransactionEvent entry) {
        if (!transactionEvents.contains(entry)) {
            transactionEvents.addElement(entry);
        }
    }

    /**
     * Add an event to transaction for the destruction of the specified object.
     */
    public void addDestroyObject(final NakedObject object) {
        add(new ClientTransactionEvent(object, ClientTransactionEvent.DELETE));
    }

    /**
     * Add an event to transaction for the adding of the specified object.
     */
    public void addMakePersistent(final NakedObject object) {
        add(new ClientTransactionEvent(object, ClientTransactionEvent.ADD));
    }

    /**
     * Add an event to transaction for the updating of the specified object.
     */
    public void addObjectChanged(final NakedObject object) {
        add(new ClientTransactionEvent(object, ClientTransactionEvent.CHANGE));
    }

    /**
     * Return all the events for the transaction.
     */
    public ClientTransactionEvent[] getEntries() {
        ClientTransactionEvent[] array = new ClientTransactionEvent[transactionEvents.size()];
        transactionEvents.copyInto(array);
        return array;
    }

    /**
     * Returns true if there are no entries, hence the transaction is not used.
     */
    public boolean isEmpty() {
        return transactionEvents.size() == 0;
    }

    public void rollback() {
        // TODO need to restore the state of all involved objects
    }
}
// Copyright (c) Naked Objects Group Ltd.
