package org.nakedobjects.nof.reflect.remote.data;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.adapter.Version;
import org.nakedobjects.noa.persist.InstancesCriteria;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.spec.NakedObjectSpecification;


public interface ObjectEncoder {
    IdentityData createIdentityData(NakedObject object);

    Data[] createParameters(NakedObjectSpecification[] parameterTypes, Naked[] parameters, KnownObjects knownObjects);

    ReferenceData createActionTarget(NakedObject target, KnownObjects knownObjects);

    /**
     * Creates an ObjectData that contains all the data for all the transient objects in the specified
     * transient object. For any referenced persistent object in the graph, only the reference is passed
     * across.
     */
    ObjectData createMakePersistentGraph(NakedObject object, KnownObjects knownObjects);

    ObjectData createGraphForChangedObject(NakedObject object, KnownObjects knownObjects);

    /**
     * Creates a a graph of ReferenceData objects (mirroring the graph of transient objects) to transfer the
     * OIDs and Versions for each object that was made persistent during the makePersistent call.
     */
    ObjectData createMadePersistentGraph(ObjectData data, NakedObject object);

    ServerActionResultData createServerActionResult(
            Naked result,
            ObjectData[] updates,
            ReferenceData[] disposed,
            ObjectData persistedTarget,
            ObjectData[] persistedParameters,
            String[] messages, String[] warnings);

    ClientActionResultData createClientActionResult(ReferenceData[] madePersistent, Version[] changedVersion, ObjectData[] updates);

    /**
     * Creates an ObjectData that contains all the data for all the objects in the graph. This allows the
     * client to receive all data it might need without having to return to the server to get referenced
     * objects.
     */
    ObjectData createCompletePersistentGraph(NakedObject object);

    Data createForResolveField(NakedObject object, String fieldName);

    /**
     * Creates an ObjectData that contains the data for the specified object, but not the data for any
     * referenced objects. For each referenced object only the reference is passed across.
     */
    ObjectData createForUpdate(NakedObject object);

    void madePersistent(NakedObject target, ObjectData persistedTarget);

    Naked restore(Data returned);

    Naked restore(Data data, KnownObjects knownObjects);

    /**
     * Returns the agreed order to transfer fields within data objects. Both remote parties need to process
     * the fields in the same order, this is that order.
     */
    NakedObjectField[] getFieldOrder(NakedObjectSpecification specification);

    ValueData createValue(NakedValue value);

    CriteriaData createCriteriaData(InstancesCriteria criteria);

    InstancesCriteria restoreCriteria(CriteriaData criteriaData);
}

// Copyright (c) Naked Objects Group Ltd.
