package org.nakedobjects.nof.reflect.security;

import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.core.reflect.Veto;
import org.nakedobjects.nof.reflect.peer.AbstractValuePeer;
import org.nakedobjects.nof.reflect.peer.ValuePeer;


/**
 * One to one authorisation decorator adds checks via the <code>AuthorisationManager</code> to
 * <code>isAuthorised</code> and <code>isAvailable</code> calls.
 */
public class ValueAuthorisation extends AbstractValuePeer {
    private final AuthorisationManager authorisationManager;

    public ValueAuthorisation(final ValuePeer local, final AuthorisationManager authorisationManager) {
        super(local);
        this.authorisationManager = authorisationManager;
    }

    public Consent isUsableForSession(final Session session) {
        if (!authorisationManager.isEditable(session, getIdentifier())) {
            return new Veto("Not permitted to edit field");
        } else {
            return Allow.DEFAULT;
        }
    }

    public boolean isVisibleForSession(final Session session) {
        return authorisationManager.isVisible(session, getIdentifier());
    }

}
// Copyright (c) Naked Objects Group Ltd.
