package org.nakedobjects.nof.reflect.spec;

import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedReference;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.annotations.Constraint;
import org.nakedobjects.noa.annotations.MandatorySemantic;
import org.nakedobjects.noa.annotations.MaxLengthAnnotation;
import org.nakedobjects.noa.annotations.RegExAnnotation;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.reflect.NakedObjectActionParameter;
import org.nakedobjects.noa.reflect.NakedObjectActionParameter.Filter;
import org.nakedobjects.noa.spec.Features;
import org.nakedobjects.noa.spec.HasType;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.persist.AllInstances;
import org.nakedobjects.nof.core.reflect.Allow;
import org.nakedobjects.nof.core.reflect.Veto;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.core.util.UnknownTypeException;
import org.nakedobjects.nof.reflect.peer.ActionParamPeer;
import org.nakedobjects.nof.reflect.peer.ActionPeer;

import java.util.Enumeration;

import org.apache.log4j.Logger;


public class ActionImpl extends AbstractNakedObjectMember implements NakedObjectAction {
    private final static Logger LOG = Logger.getLogger(ActionImpl.class);

    public static NakedObjectAction.Type getType(String type) {
        NakedObjectAction.Type[] types = new NakedObjectAction.Type[] { NakedObjectAction.DEBUG, NakedObjectAction.EXPLORATION,
                NakedObjectAction.USER };
        for (int i = 0; i < types.length; i++) {
            if (types[i].getName().equals(type)) {
                return types[i];
            }
        }
        throw new IllegalArgumentException();
    }

    private final ActionPeer reflectiveAdapter;

    public ActionImpl(final String methodId, final ActionPeer actionPeer) {
        super(methodId);
        this.reflectiveAdapter = actionPeer;
    }

    private Naked adaptedPojo(Object pojo, NakedObjectSpecification type) {
        if (pojo == null) {
            return null;
        } else {
            if (type.getType() == NakedObjectSpecification.VALUE) {
                NakedValue valueAdapter = NakedObjectsContext.getObjectLoader().createAdapterForValue(pojo);
                // TODO set up mask
                return valueAdapter;
            } else {
                return NakedObjectsContext.getObjectLoader().getAdapterForElseCreateAdapterForTransient(pojo);
            }
        }
    }

    public boolean[] canParametersWrap() {
        return reflectiveAdapter.canParametersWrap();
    }

    public String debugData() {
        DebugString debugString = new DebugString();
        debugString.indent();
        debugString.indent();
        debugString.indent();
        reflectiveAdapter.debugData(debugString);
        return debugString.toString();
    }

    public Naked execute(final NakedReference object, final Naked[] parameters) {
        LOG.debug("execute action " + object + "." + getId());
        Naked[] params = realParameters(object, parameters);
        NakedReference target = realTarget(object);
        Naked result = reflectiveAdapter.execute(target, params);
        return result;
    }

    private Naked[] realParameters(final NakedReference target, final Naked[] parameters) {
        Naked[] params;
        if (parameters == null) {
            params = isContributedMethodWithSuitableParameter() ? new Naked[] { target } : new Naked[0];
        } else {
            params = parameters;
        }
        return params;
    }

    public NakedObjectAction[] getActions() {
        return new NakedObjectAction[0];
    }

    public Naked[] getDefaultParameterValues(NakedReference target) {
        NakedReference realTarget = realTarget(target);
        Object[] defaults = reflectiveAdapter.getParameterDefaults(realTarget);
        int parameterCount = getParameterCount();
        Naked[] parameters = new Naked[parameterCount];
        if (defaults != null) {
            NakedObjectSpecification[] parameterTypes = getParameterTypes();
            for (int i = 0; i < parameterCount; i++) {
                parameters[i] = adaptedPojo(defaults[i], parameterTypes[i]);
            }
        }
        if (isContributedMethodWithSuitableParameter() && target != null) {
            NakedObjectSpecification[] parameterTypes = getParameterTypes();
            for (int i = 0; i < parameterCount; i++) {
                if (target.getSpecification().isOfType(parameterTypes[i])) {
                    parameters[i] = target;
                }
            }
        }
        return parameters;
    }

    public NakedReference realTarget(final NakedReference target) {
        if (!isOnInstance()) {
            // class actions have no target object
            return null;
        } else if (target == null) {
            return findService();
        } else if (Features.isService(target.getSpecification())) {
            return target;
        } else if (isContributedMethodWithSuitableParameter()) {
            return findService();
        } else {
            return target;
        }
    }

    private NakedObject findService() {
        Object[] services = NakedObjectsContext.getObjectLoader().getServices();
        for (int i = 0; i < services.length; i++) {
            NakedObject serviceAdapter = NakedObjectsContext.getObjectLoader().getAdapterFor(services[i]);
            if (serviceAdapter.getSpecification() == getOnType()) {
                return serviceAdapter;
            }
        }
        throw new NakedObjectRuntimeException("failed to find service for action " + this.getName());
    }

    public String getDescription() {
        String description = reflectiveAdapter.getDescription();
        return description == null ? "" : description;
    }

    public Object getExtension(final Class cls) {
        return reflectiveAdapter.getExtension(cls);
    }

    public Class[] getExtensions() {
        return reflectiveAdapter.getExtensions();
    }

    public String getHelp() {
        String help = reflectiveAdapter.getHelp();
        return help == null ? "" : help;
    }

    /**
     * Return the default label for this member. This is based on the name of this member.
     * 
     * @see #getId()
     */
    public String getName() {
        String label = reflectiveAdapter.getName();

        // REVIEW example of using Dan's annotation classes
        // NamedAnnotation an = (NamedAnnotation) getExtension(NamedAnnotation.class);
        // String label = an == null ? null : (an.value());

        String name = label == null ? defaultName : label;
        return name; // + " (" + getOnType().getShortName() + ")";
    }

    public NakedObjectSpecification getOnType() {
        return reflectiveAdapter.getOnType();
    }

    public Naked[][] getOptions(final NakedReference target) {
        NakedReference realTarget = realTarget(target);
        Object[][] optionPojos = reflectiveAdapter.getParameterOptions(realTarget);
        int noParameters = getParameterCount();
        if (optionPojos == null) {
            optionPojos = new Object[noParameters][];
        }

        NakedObjectSpecification[] parameterTypes = reflectiveAdapter.getParameterTypes();
        Naked[][] options = new Naked[noParameters][];
        for (int i = 0; i < noParameters; i++) {
            if (optionPojos[i] != null) {
                if (parameterTypes[i].getType() == NakedObjectSpecification.VALUE) {
                    options[i] = new NakedValue[optionPojos[i].length];
                } else {
                    options[i] = new NakedObject[optionPojos[i].length];
                }
                for (int j = 0; j < optionPojos[i].length; j++) {
                    options[i][j] = adaptedPojo(optionPojos[i][j], parameterTypes[i]);
                }
            } else if (Features.isBoundedSet(parameterTypes[i])) {
                NakedCollection instances = NakedObjectsContext.getObjectPersistor().findInstances(
                        new AllInstances(parameterTypes[i], false));
                options[i] = new NakedObject[instances.size()];
                int j = 0;
                for (Enumeration e = instances.elements(); e.hasMoreElements();) {
                    NakedObject adapter = (NakedObject) e.nextElement();
                    options[i][j++] = adapter;
                }
            } else if (parameterTypes[i].getType() == NakedObjectSpecification.VALUE) {
                options[i] = new NakedValue[0];
            } else if (parameterTypes[i].getType() == NakedObjectSpecification.OBJECT) {
                options[i] = null;
            } else {
                throw new UnknownTypeException(parameterTypes[i]);
            }
        }

        return options;
    }

    public int getParameterCount() {
        return reflectiveAdapter.getParameterCount();
    }

    /**
     * Build lazily by {@link #getParameters()}.
     */
    private NakedObjectActionParameter[] parameters;

    // REVIEW is it a good idea to lazily load this as it a static object and more than one thread might call
    // it
    public synchronized NakedObjectActionParameter[] getParameters() {
        if (this.parameters == null) {
            this.parameters = new NakedObjectActionParameter[getParameterCount()];
            final ActionParamPeer[] paramPeers = reflectiveAdapter.getParameters();

            NakedObjectSpecification[] parameterTypes = getParameterTypes();
            String[] parameterNames = getParameterNames();
            String[] parameterDescriptions = getParameterDescriptions();
            boolean[] optionalParameters = getOptionalParameters();

            int[] parameterNoLines = getParameterNoLines();
            boolean[] canParametersWrap = canParametersWrap();
            int[] parameterMaxLengths = getParameterMaxLengths();
            int[] parameterTypicalLengths = getParameterTypicalLengths();

            for (int i = 0; i < getParameterCount(); i++) {
                if (parameterTypes[i].getType() == NakedObjectSpecification.VALUE) {
                    parameters[i] = new DefaultValueActionParameter(i, this, paramPeers[i], parameterTypes[i], parameterNames[i],
                            parameterDescriptions[i], optionalParameters[i], parameterTypicalLengths[i], parameterMaxLengths[i],
                            parameterNoLines[i], canParametersWrap[i]);
                } else if (parameterTypes[i].getType() == NakedObjectSpecification.OBJECT) {
                    parameters[i] = new DefaultOneToOneActionParameter(i, this, paramPeers[i], parameterTypes[i],
                            parameterNames[i], parameterDescriptions[i], optionalParameters[i]);
                } else if (parameterTypes[i].getType() == NakedObjectSpecification.COLLECTION) {
                    throw new UnknownTypeException("collections not supported as parameters");
                }

            }
        }

        return parameters;
    }

    public NakedObjectActionParameter[] getParameters(Filter filter) {
        NakedObjectActionParameter[] allParameters = getParameters();

        NakedObjectActionParameter[] selectedParameters = new NakedObjectActionParameter[allParameters.length];
        int v = 0;
        for (int i = 0; i < allParameters.length; i++) {
            if (filter.accept(allParameters[i])) {
                selectedParameters[v++] = allParameters[i];
            }
        }

        NakedObjectActionParameter[] parameters = new NakedObjectActionParameter[v];
        System.arraycopy(selectedParameters, 0, parameters, 0, v);
        return parameters;
    }

    public String[] getParameterDescriptions() {
        String[] descriptions = reflectiveAdapter.getParameterDescriptions();
        if (descriptions == null) {
            int len = getParameterTypes().length;
            descriptions = new String[len];
            for (int i = 0; i < len; i++) {
                descriptions[i] = "";
            }
        }
        return descriptions;
    }

    public int[] getParameterMaxLengths() {
        return reflectiveAdapter.getParameterMaxLengths();
    }

    public String[] getParameterNames() {
        String[] names = reflectiveAdapter.getParameterNames();
        NakedObjectSpecification[] parameterTypes = getParameterTypes();
        if (names == null) {
            names = new String[parameterTypes.length];
            for (int i = 0; i < parameterTypes.length; i++) {
                names[i] = parameterTypes[i].getSingularName();
            }
        } else {
            for (int i = 0; i < names.length; i++) {
                names[i] = names[i] == null ? parameterTypes[i].getSingularName() : names[i];
            }
        }
        return names;
    }

    public int[] getParameterNoLines() {
        return reflectiveAdapter.getParameterNoLines();
    }

    public NakedObjectSpecification[] getParameterTypes() {
        return reflectiveAdapter.getParameterTypes();
    }

    public int[] getParameterTypicalLengths() {
        return reflectiveAdapter.getParameterTypicalLengths();
    }

    public boolean[] getOptionalParameters() {
        boolean[] optional = reflectiveAdapter.getOptionalParameters();
        if (optional == null) {
            optional = new boolean[getParameterCount()];
        }
        return optional;
    }

    public NakedObjectSpecification getReturnType() {
        return reflectiveAdapter.getReturnType();
    }

    public NakedObjectAction.Target getTarget() {
        return reflectiveAdapter.getTarget();
    }

    public NakedObjectAction.Type getType() {
        return reflectiveAdapter.getType();
    }

    /**
     * Returns true if the represented action returns something, else returns false.
     */
    public boolean hasReturn() {
        return getReturnType() != null;
    }

    public boolean isOnInstance() {
        return reflectiveAdapter.isOnInstance();
    }

    public boolean isContributedMethodWithSuitableParameter() {
        if (Features.isService(getOnType()) && getParameterCount() > 0) {
            NakedObjectActionParameter[] params = getParameters();
            for (int i = 0; i < params.length; i++) {
                if (params[i].isObject()) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Checks declarative constraints, and then checks imperatively.
     */
    public Consent isParameterSetValid(final NakedReference object, final Naked[] parameters) {
        Consent declarativeConsent = isParameterSetValidDeclaratively(object, parameters);
        if (declarativeConsent.isVetoed()) {
            return declarativeConsent;
        }
        return isParameterSetValidImperatively(object, parameters);
    }

    public Consent isParameterSetValidDeclaratively(final NakedReference object, final Naked[] parameters) {
        Naked[] params = realParameters(object, parameters);
        for (int i = 0; i < params.length; i++) {
            Naked param = params[i];
            Consent consent = checkParamMandatory(i, param);
            if (consent.isVetoed()) {
                return consent;
            }
            consent = checkParamMaxLength(i, param);
            if (consent.isVetoed()) {
                return consent;
            }
            consent = checkParamRegEx(i, param);
            if (consent.isVetoed()) {
                return consent;
            }
        }
        return Allow.DEFAULT;
    }

    private Consent checkParamMandatory(int position, Naked argNaked) {

        NakedObjectActionParameter parameter = getParameter(position);
        Constraint mandatoryConstraint = (MandatorySemantic) parameter.getExtension(MandatorySemantic.class);
        if (mandatoryConstraint == null) {
            return Allow.DEFAULT;
        }

        int parameterType = parameter.getSpecification().getType();
        if (parameterType != HasType.OBJECT && parameterType != HasType.VALUE) {
            return Allow.DEFAULT;
        }

        if (mandatoryConstraint.violatedBy(argNaked)) {
            return new Veto(mandatoryConstraint.createExceptionFor(getId(), argNaked));
        }

        return Allow.DEFAULT;
    }

    private Consent checkParamMaxLength(int position, Naked argNaked) {
        NakedObjectActionParameter parameter = getParameter(position);
        Constraint maxLengthConstraint = (MaxLengthAnnotation) parameter.getExtension(MaxLengthAnnotation.class);
        if (maxLengthConstraint == null) {
            return Allow.DEFAULT;
        }
        if (maxLengthConstraint.violatedBy(argNaked)) {
            return new Veto(maxLengthConstraint.createExceptionFor(getId(), argNaked));
        }
        return Allow.DEFAULT;
    }

    private Consent checkParamRegEx(int position, Naked argNaked) {
        NakedObjectActionParameter parameter = getParameter(position);
        Constraint regExConstraint = (RegExAnnotation) parameter.getExtension(RegExAnnotation.class);
        if (regExConstraint == null) {
            return Allow.DEFAULT;
        }
        if (regExConstraint.violatedBy(argNaked)) {
            return new Veto(regExConstraint.createExceptionFor(getId(), argNaked));
        }
        return Allow.DEFAULT;
    }

    private NakedObjectActionParameter getParameter(int position) {
        NakedObjectActionParameter[] parameters = getParameters();
        if (position >= parameters.length) {
            throw new IllegalArgumentException("getParameter(int): only " + parameters.length + " parameters, position="
                    + position);
        }
        return parameters[position];
    }

    public Consent isParameterSetValidImperatively(NakedReference object, Naked[] parameters) {
        Naked[] params = realParameters(object, parameters);
        NakedReference target = realTarget(object);
        return reflectiveAdapter.isParameterSetValidImperatively(target, params);
    }

    public Consent isUsableDeclaratively() {
        return reflectiveAdapter.isUsableDeclaratively();
    }

    public Consent isUsableForSession() {
        return reflectiveAdapter.isUsableForSession(NakedObjectsContext.getSession());
    }

    public Consent isUsable(final NakedReference target) {
        return reflectiveAdapter.isUsable(realTarget(target));
    }

    public boolean isVisibleDeclaratively() {
        return reflectiveAdapter.isVisibleDeclaratively();
    }

    public boolean isVisibleForSession() {
        return reflectiveAdapter.isVisibleForSession(NakedObjectsContext.getSession());
    }

    public boolean isVisible(final NakedReference target) {
        return reflectiveAdapter.isVisible(realTarget(target));
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Action [");
        sb.append(super.toString());
        sb.append(",type=");
        sb.append(getType());
        sb.append(",returns=");
        sb.append(getReturnType());
        sb.append(",parameters={");
        for (int i = 0; i < getParameterTypes().length; i++) {
            if (i > 0) {
                sb.append(",");
            }
            sb.append(getParameterTypes()[i]);
        }
        sb.append("}]");
        return sb.toString();
    }

    /**
     * Always returns <tt>null</tt>.
     */
    public NakedObjectSpecification getSpecification() {
        return null;
    }

}
// Copyright (c) Naked Objects Group Ltd.
