package org.nakedobjects.nof.reflect.spec;

import java.util.ArrayList;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.noa.spec.NakedCollectionSpecification;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.reflect.Veto;


public class CollectionSpecification extends AbstractSpecification implements NakedCollectionSpecification {
    private final Class elementClass;

    public CollectionSpecification(String className, Class elementClass) {
        // to deal with generic collections remove the following prefix
        String prefix = "java.util.Arrays$";
        if (className.startsWith(prefix)) {
            this.className = "java.util." + className.substring(prefix.length());
        } else {
            this.className = className;
        }
        this.elementClass = elementClass;
    }

    public NakedObjectSpecification getElementType() {
        return NakedObjectsContext.getReflector().loadSpecification(elementClass);
    }

    public int getType() {
        return COLLECTION;
    }

    public Consent isPersistable(NakedObject transientObject) {
        return Veto.DEFAULT;
    }

    public Persistable persistable() {
        return Persistable.TRANSIENT;
    }

    public void introspect(ReflectionPeerBuilder builder) {
        fields = new NakedObjectField[0];
    }

    public NakedObjectField getField(String name) {
        return null;
    }

    public String getPluralName() {
        return "Collections";
    }

    public String getShortName() {
        return "Collection";
    }

    public String getSingularName() {
        return "Collection";
    }

    public String getDescription() {
        return "Collection";
    }

    public String getTitle(NakedObject object) {
        return "Collection";
    }

    public Object getDefaultValue() {
        return new ArrayList();
    }

}

// Copyright (c) Naked Objects Group Ltd.
