package org.nakedobjects.nof.reflect.spec;

import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.Persistable;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.NakedObjectAction;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.nof.core.reflect.Veto;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.core.util.NameConvertor;


/**
 * A simple implementation used for objects that have no members (fields or actions). Used for primitives and
 * as a fall-back if no specification can be otherwise generated.
 */
public class NoMemberSpecification extends AbstractSpecification {
    private final String name;

    public NoMemberSpecification(final String className) {
        this.className = className;
        this.name = NameConvertor.simpleName(className.substring(className.lastIndexOf('.') + 1));
    }

    public void debugData(DebugString debug) {
        debug.append("There are no reflective actions");
    }

    public String debugTitle() {
        return "NO Member Specification";
    }

    public NakedObjectField getField(final String name) {
        return null;
    }

    public NakedObjectAction[] getObjectActions(final NakedObjectAction.Type type) {
        return new NakedObjectAction[0];
    }

    public String getPluralName() {
        return name;
    }

    public String getShortName() {
        return name;
    }

    public String getSingularName() {
        return name;
    }

    public String getDescription() {
        return name;
    }

    public String getTitle(final NakedObject naked) {
        return "no title";
    }

    public void introspect(ReflectionPeerBuilder builder) {
        fields = new NakedObjectField[0];
        superClassSpecification = null;
    }

    public Consent isPersistable(NakedObject transientObject) {
        return Veto.DEFAULT;
    }

    public Persistable persistable() {
        return Persistable.TRANSIENT;
    }
    
}
// Copyright (c) Naked Objects Group Ltd.
