package org.nakedobjects.nof.reflect.spec;

import org.nakedobjects.noa.adapter.Naked;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.NakedReference;
import org.nakedobjects.noa.adapter.NakedValue;
import org.nakedobjects.noa.reflect.Consent;
import org.nakedobjects.noa.reflect.Formatter;
import org.nakedobjects.noa.reflect.ValueAssociation;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.util.DebugString;
import org.nakedobjects.nof.core.util.ToString;
import org.nakedobjects.nof.reflect.peer.ValuePeer;


public class ValueAssociationImpl extends AbstractNakedObjectField implements ValueAssociation {
    private final ValuePeer reflectiveAdapter;

    public ValueAssociationImpl(
            final String fieldId,
            final NakedObjectSpecification specification,
            final ValuePeer association) {
        super(fieldId, specification);
        this.reflectiveAdapter = association;
    }

    public boolean canClear() {
        return reflectiveAdapter.canClear();
    }
    
    public boolean canWrap() {
        return reflectiveAdapter.canWrap();
    }
    
    public void clearValue(final NakedObject inObject) {
        reflectiveAdapter.clearValue(inObject);
    }

    public String debugData() {
        DebugString debugString = new DebugString();
        debugString.indent();
        debugString.indent();
        reflectiveAdapter.debugData(debugString);
        return debugString.toString();
    }

    public Naked get(final NakedObject fromObject) {
        return reflectiveAdapter.getValue(fromObject);
    }

    public String getBusinessKeyName() {
        return reflectiveAdapter.getBusinessKeyName();
    }

    public String getDescription() {
        return reflectiveAdapter.getDescription();
    }

    public Object getExtension(final Class cls) {
        return reflectiveAdapter.getExtension(cls);
    }
    
    public Class[] getExtensions() {
        return reflectiveAdapter.getExtensions();
    }

    public Formatter getFormatter() {
        return reflectiveAdapter.getFormatter();
    }

    public String getHelp() {
        return reflectiveAdapter.getHelp();
    }
    
    public int getMaximumLength() {
        return reflectiveAdapter.getMaximumLength();
    }
    
    /**
     * Return the default label for this member. This is based on the name of this member.
     * 
     * @see #getId()
     */
    public String getName() {
        String label = reflectiveAdapter.getName();
        return label == null ? defaultName : label;
    }
    
    public int getNoLines() {
        return reflectiveAdapter.getNoLines();
    }

    public Naked[] getOptions(NakedObject target) {
        Object[] pojoOptions = reflectiveAdapter.getOptions(target);
        if (pojoOptions != null) {
            Naked[] options =  new Naked[pojoOptions.length];
            for (int i = 0; i < options.length; i++) {
                options[i] = NakedObjectsContext.getObjectLoader().createAdapterForValue(pojoOptions[i]);
            }
            return options;
        } else {
            return null;
        }
    }
    
    public int getTypicalLineLength() {
        return reflectiveAdapter.getTypicalLineLength();
    }

    public void initValue(final NakedObject inObject, final NakedValue associate) {
        if (readWrite()) {
            reflectiveAdapter.initValue(inObject, associate);
        }
    }

    public boolean isPersisted() {
        return reflectiveAdapter.isPersisted();
    }

    public boolean isEmpty(final NakedObject inObject) {
        return reflectiveAdapter.isEmpty(inObject);
    }

    public boolean isMandatory() {
        return reflectiveAdapter.isMandatory();
    }
    
    public boolean isOptionEnable() {
        return reflectiveAdapter.isOptionEnabled();
    }

    public boolean isVisibleDeclaratively() {
        return reflectiveAdapter.isVisibleDeclaratively();
    }
    
    public boolean isVisibleForSession() {
        return reflectiveAdapter.isVisibleForSession(NakedObjectsContext.getSession());
    }
    
    public boolean isVisible(final NakedReference target) {
        return reflectiveAdapter.isVisible(target);
    }

    public Consent isUsableDeclaratively() {
        return reflectiveAdapter.isUsableDeclaratively();
    }

    public Consent isUsableForSession() {
        return reflectiveAdapter.isUsableForSession(NakedObjectsContext.getSession());
    }

    public Consent isUsable(final NakedReference target) {
        return reflectiveAdapter.isUsable(target);
    }

    public boolean isValue() {
        return true;
    }
    
    public Consent isValueValid(final NakedObject inObject, final NakedValue value) {
        return reflectiveAdapter.isValueValid(inObject, value);
    }


    private boolean readWrite() {
        return reflectiveAdapter.isPersisted();
    }
    
    public Object getDefault(
            NakedObject nakedObject) {
        return reflectiveAdapter.getDefault(nakedObject);
    }

    public void toDefault(final NakedObject inObject) {
        NakedValue defaultValue = (NakedValue) getDefault(inObject);
        if (defaultValue != null) {
            initValue(inObject, defaultValue);
        }
    }

    public void setValue(final NakedObject inObject, final NakedValue value) {
        if (readWrite()) {
            reflectiveAdapter.setValue(inObject, value);
        }
    }

    public String toString() {
        ToString str = new ToString(this);
        str.append(super.toString());
        str.setAddComma();
        str.append("persisted", isPersisted());
        str.append("type", getSpecification().getShortName());
        return str.toString();
    }

}
// Copyright (c) Naked Objects Group Ltd.
