package org.nakedobjects.nof.reflect.transaction;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.reflect.peer.AbstractOneToManyPeer;
import org.nakedobjects.nof.reflect.peer.OneToManyPeer;


public class OneToManyTransaction extends AbstractOneToManyPeer {
    private final static Logger LOG = Logger.getLogger(OneToManyTransaction.class);

    public OneToManyTransaction(final OneToManyPeer peer) {
        super(peer);
    }

    public void addAssociation(final NakedObject inObject, final NakedObject associate) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.addAssociation(inObject, associate);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.addAssociation(inObject, associate);
        }
    }

    public void removeAllAssociations(final NakedObject inObject) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.removeAllAssociations(inObject);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.removeAllAssociations(inObject);

        }
    }

    private void abort(final NakedObjectPersistor objectManager) {
        LOG.info("exception executing " + getIdentifier() + ", aborting transaction");
        try {
            objectManager.abortTransaction();
        } catch (Exception e2) {
            LOG.error("failure during abort", e2);
        }
    }

    public void removeAssociation(final NakedObject inObject, final NakedObject associate) {
        NakedObjectPersistor objectManager = NakedObjectsContext.getObjectPersistor();
        if (inObject.getResolveState().isPersistent()) {
            try {
                objectManager.startTransaction();
                super.removeAssociation(inObject, associate);
                objectManager.saveChanges();
                objectManager.endTransaction();
            } catch (RuntimeException e) {
                abort(objectManager);
                throw e;
            }
        } else {
            super.removeAssociation(inObject, associate);
        }

    }
}
// Copyright (c) Naked Objects Group Ltd.
